"""Convert textcat annotation from JSONL to spaCy v3 .spacy format."""
import srsly
import typer
from pathlib import Path
from rich.progress import track

import spacy
from spacy.tokens import DocBin


def convert(lang: str, input_path: Path, output_path: Path, tags_of_interest: str):
    """Convert textcat annotation from JSONL to spaCy v3 .spacy format."""
    tags_of_interest = tags_of_interest.split(",")
    nlp = spacy.blank(lang)
    db = DocBin()
    for line in track(list(srsly.read_jsonl(input_path)), "Creating .spacy file."):
        doc = nlp.make_doc(line["text"])
        doc.cats = {k: line["cats"][k] for k in tags_of_interest}
        db.add(doc)
    db.to_disk(output_path)


if __name__ == "__main__":
    typer.run(convert)



import numpy as np
import pandas as pd
import altair as alt
from clumper import Clumper
from sklearn.metrics import precision_score, recall_score, accuracy_score


def create_density_chart(dataf):
    """Creates a density chart."""
    return (
        alt.Chart(dataf)
        .transform_density(
            "score", as_=["score", "density"], groupby=["group"], bandwidth=0.05
        )
        .mark_area(opacity=0.7)
        .encode(
            x="score:Q",
            y="density:Q",
            color="group:N",
            tooltip=["score:Q", "density:Q", "group:N"],
        )
        .properties(title="Density of Examples")
    )


def create_line_chart(dataf, x_col, y_col, group_col):
    """Based on https://altair-viz.github.io/gallery/multiline_tooltip.html"""
    # Create a selection that chooses the nearest point & selects based on x-value
    nearest = alt.selection(
        type="single", nearest=True, on="mouseover", fields=[x_col], empty="none"
    )

    # The basic line
    line = (
        alt.Chart(dataf)
        .mark_line(interpolate="basis")
        .encode(x=f"{x_col}:Q", y=f"{y_col}:Q", color=f"{group_col}:N")
    )

    # Transparent selectors across the chart. This is what tells us
    # the x-value of the cursor
    selectors = (
        alt.Chart(dataf)
        .mark_point()
        .encode(
            x=f"{x_col}:Q",
            opacity=alt.value(0),
        )
        .add_selection(nearest)
    )

    # Draw points on the line, and highlight based on selection
    points = line.mark_point().encode(
        opacity=alt.condition(nearest, alt.value(1), alt.value(0))
    )

    # Draw text labels near the points, and highlight based on selection
    text = line.mark_text(align="left", dx=5, dy=-5).encode(
        text=alt.condition(nearest, f"{y_col}:Q", alt.value(" "))
    )

    # Draw a rule at the location of the selection
    rules = (
        alt.Chart(dataf)
        .mark_rule(color="gray")
        .encode(
            x=f"{x_col}:Q",
        )
        .transform_filter(nearest)
    )

    # Put the five layers into a chart and bind the data
    return alt.layer(line, selectors, points, rules, text).properties(
        title="Threshold Effect"
    )


def make_plots(orig_data, pred_data, tag):
    """Creates a 2x2 grid of charts for debugging."""
    orig_dictlist = [{"text": _.text, "labels": _.cats} for _ in orig_data]
    combined = [{**clump, "pred": d.cats} for clump, d in zip(orig_dictlist, pred_data)]

    conf_data = (
        Clumper(combined)
        .mutate(
            confidence=lambda d: d["pred"][tag], hit=lambda d: d["labels"][tag] == 1.0
        )
        .collect()
    )

    # Prepare the density chart
    x_hit = np.array([d["confidence"] for d in conf_data if d["hit"]])
    x_miss = np.array([d["confidence"] for d in conf_data if not d["hit"]])

    df_density = pd.concat(
        [
            pd.DataFrame({"score": x_hit}).assign(group=lambda d: tag),
            pd.DataFrame({"score": x_miss}).assign(group=lambda d: f"not-{tag}"),
        ]
    )

    p_density = create_density_chart(df_density)

    # Prepare the scores chart
    xs = np.linspace(0.01, 0.95, 100)
    precisions = np.zeros(xs.shape)
    recalls = np.zeros(xs.shape)
    accuracies = np.zeros(xs.shape)

    y_true = np.array([d["hit"] for d in conf_data])
    confs = np.array([d["confidence"] for d in conf_data])

    for i, x in enumerate(xs):
        y_pred = confs > x
        precisions[i] = precision_score(y_true, y_pred, zero_division=0)
        recalls[i] = recall_score(y_true, y_pred, zero_division=0)
        accuracies[i] = accuracy_score(y_true, y_pred)

    precisions = np.round(precisions, 4)
    recalls = np.round(recalls, 4)
    accuracies = np.round(accuracies, 4)

    df_scores = pd.concat(
        [
            pd.DataFrame(
                {
                    "threshold": xs,
                    "score": precisions,
                    "group": ["precision" for _ in xs],
                }
            ),
            pd.DataFrame(
                {"threshold": xs, "score": recalls, "group": ["recall" for _ in xs]}
            ),
            pd.DataFrame(
                {
                    "threshold": xs,
                    "score": accuracies,
                    "group": ["accuracy" for _ in xs],
                }
            ),
        ]
    )

    p_scores = create_line_chart(
        df_scores, x_col="threshold", y_col="score", group_col="group"
    )
    return p_density, p_scores



import pathlib
import warnings
from pkg_resources import resource_filename

import spacy
from spacy.tokens import DocBin
import typer
import altair as alt
from rich.progress import track
from rich.console import Console
from jinja2 import Environment, select_autoescape, FileSystemLoader

from accuracy._charts import make_plots
from accuracy import __version__


app = typer.Typer(
    name="accuraCy",
    add_completion=False,
    help="It's pronounced 'accura-see'. For spaCy models.",
)


@app.command()
def report(
    model_path: pathlib.Path = typer.Argument(None, help="Path to spaCy model"),
    train_path: pathlib.Path = typer.Argument(None, help="Path to training data"),
    dev_path: pathlib.Path = typer.Argument(None, help="Path to development data"),
    folder_out: pathlib.Path = typer.Argument(
        "reports", help="Output folder for reports"
    ),
    classes: str = typer.Option(None, help="Comma-separated string of classes to use"),
):
    """Generate a model report."""
    console = Console()
    console.print(f"Loading model at {model_path}")
    nlp = spacy.load(model_path)
    tags_of_interest = list(nlp("categories").cats.keys())
    if classes:
        for classname in classes.split(","):
            if classname not in tags_of_interest:
                console.print(
                    f"[red]Warning! '{classname}' label not found in model. Skipping... [/red]"
                )
        tags_of_interest = [c for c in tags_of_interest if c in classes.split(",")]

    orig_train_docbin = list(DocBin().from_disk(train_path).get_docs(nlp.vocab))
    orig_valid_docbin = list(DocBin().from_disk(dev_path).get_docs(nlp.vocab))

    # At the time of writing this the thinc library has an annoying warning
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore")
        console.print("Running model on training data...")
        pred_train_clump = list(nlp.pipe([_.text for _ in orig_train_docbin]))
        console.print("Running model on development data...")
        pred_valid_clump = list(nlp.pipe([_.text for _ in orig_valid_docbin]))

        alt.data_transformers.disable_max_rows()

        if not folder_out.exists():
            folder_out.mkdir(parents=True)

        for tag in track(tags_of_interest, "Generating Charts"):
            # Prepare plotting objects
            p1, p2 = make_plots(orig_train_docbin, pred_train_clump, tag=tag)
            p3, p4 = make_plots(orig_valid_docbin, pred_valid_clump, tag=tag)

            # Render the density charts
            with alt.themes.enable("opaque"):
                json_hist = (
                    p1.properties(title="Train") | p3.properties(title="Dev")
                ).to_json()
            pathlib.Path(folder_out, f"{tag}-hist.json").write_text(json_hist)

            # Render the acc/precision/recall charts.
            with alt.themes.enable("opaque"):
                json_scores = (
                    p2.properties(title="Train") | p4.properties(title="Dev")
                ).to_json()
            pathlib.Path(folder_out, f"{tag}-scores.json").write_text(json_scores)

    env = Environment(
        loader=FileSystemLoader(resource_filename("accuracy", "templates")),
        autoescape=select_autoescape(["html", "xml"]),
    )

    dashboard = env.get_template("template.html").render(tags=tags_of_interest)
    pathlib.Path(folder_out, "index.html").write_text(dashboard)
    console.print("Done! You can view the report via;\n")
    console.print(f"python -m http.server --directory {folder_out} PORT \n")


@app.command()
def version():
    """Show version number."""
    print(__version__)


if __name__ == "__main__":
    app()



from pathlib import Path
from typer.testing import CliRunner

from accuracy import __version__
from accuracy.__main__ import app

runner = CliRunner()


def test_version():
    """Confirm that we can get the version out."""
    result = runner.invoke(app, ["version"])
    assert result.exit_code == 0
    assert __version__ in result.stdout


def test_smoke_test(tmpdir):
    """Merely a smoke test to make sure that 'it can run'."""
    result = runner.invoke(
        app,
        [
            "report",
            "training/model-best/",
            "tests/data/train.spacy",
            "tests/data/train.spacy",
            str(tmpdir),
        ],
    )
    assert result.exit_code == 0

    # This model is taught to predict two classes.
    # We expect two charts, ie. two json files, per class.
    assert len(list(Path(tmpdir).glob("*.json"))) == 4



import pathlib
from setuptools import setup, find_packages


base_packages = [
    "spacy>=3.0.0",
    "altair>=4.2.0",
    "clumper>=0.2.15",
    "rich>=10.3.0",
    "scikit-learn>=1.0.0",
    "typer>=0.3.0",
    "Jinja2>=3.1.1",
]

dev_packages = [
    "flake8>=3.6.0",
    "pytest>=4.0.2",
    "pre-commit>=2.17.0",
    "interrogate>=1.5.0",
    "black>=21.0.0",
]

setup(
    name="accuracy",
    version="0.1.1",
    author="Vincent D. Warmerdam",
    packages=find_packages(exclude=["notebooks", "docs"]),
    description="It's pronounced accura-see. For spaCy models.",
    long_description=pathlib.Path("readme.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://koaning.github.io/accuracy/",
    project_urls={
        "Documentation": "https://koaning.github.io/accuracy/",
        "Source Code": "https://github.com/koaning/accuracy/",
        "Issue Tracker": "https://github.com/koaning/accuracy/issues",
    },
    install_requires=base_packages,
    extras_require={"base": base_packages, "dev": base_packages + dev_packages},
    package_data={"accuracy": ["templates/*.html", "templates/*.js"]},
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)
